@@ -17,10 +17,28 @@ __all__ = ['Date', 'CellRendererDate', 'Time', 'CellRendererTime', 'DateTime']
 _ = gettext.gettext
 
 
+def _fix_format(format_):
+    if '%Y' in format_:
+        if (datetime.date.min.strftime('%Y') != '0001'
+                and datetime.date.min.strftime('%4Y') == '0001'):
+            format_ = format_.replace('%Y', '%4Y')
+    return format_
+
+
 def date_parse(text, format_='%x'):
+    try:
+        return datetime.datetime.strptime(text, format_)
+    except ValueError:
+        pass
     formatted_date = datetime.date(1988, 7, 16).strftime(format_)
-    dayfirst = formatted_date.index('16') == 0
-    monthfirst = formatted_date.index('7') <= 1
+    try:
+        dayfirst = formatted_date.index('16') == 0
+    except ValueError:
+        dayfirst = False
+    try:
+        monthfirst = formatted_date.index('7') <= 1
+    except ValueError:
+        monthfirst = False
     yearfirst = not dayfirst and not monthfirst
     return parse(text, dayfirst=dayfirst, yearfirst=yearfirst, ignoretz=True)
 
@@ -176,7 +194,7 @@ class Date(gtk.Entry):
             self.__date = value
             self.update_label()
         elif prop.name == 'format':
-            self.__format = value
+            self.__format = _fix_format(value)
             self.update_label()
 
     def do_get_property(self, prop):
@@ -205,7 +223,7 @@ class CellRendererDate(gtk.CellRendererText):
 
     def do_set_property(self, prop, value):
         if prop.name == 'format':
-            self.__format = value
+            self.__format = _fix_format(value)
             return
         gtk.CellRendererText.set_property(self, prop, value)
 
@@ -326,7 +344,7 @@ class Time(gtk.ComboBoxEntry):
             self.__time = value
             self.update_label()
         elif prop.name == 'format':
-            self.__format = value
+            self.__format = _fix_format(value)
             self.update_label()
             self.update_model()
 
@@ -356,7 +374,7 @@ class CellRendererTime(gtk.CellRendererText):
 
     def do_set_property(self, prop, value):
         if prop.name == 'format':
-            self.__format = value
+            self.__format = _fix_format(value)
             return
         gtk.CellRendererText.set_property(self, prop, value)
 
@@ -423,6 +441,10 @@ class DateTime(gtk.HBox):
         self.__time.connect('time-changed',
             lambda e: self.emit('datetime-changed'))
 
+    def parse(self):
+        self.__date.parse()
+        self.__time.parse()
+
     def do_set_property(self, prop, value):
         if prop.name == 'value':
             self.__date.props.value = value
