@@ -257,7 +257,9 @@ class Screen(SignalEvent):
                 self.context_screen.display(set_cursor=True)
                 return False
             context = self.local_context
-            context.update(self.context_screen.get_on_change_value())
+            screen_context = self.context_screen.get_on_change_value()
+            screen_context.pop('id')
+            context.update(screen_context)
             self.new_group(context)
 
         domain = self.search_domain(search_string, True)
@@ -885,8 +887,6 @@ class Screen(SignalEvent):
         return json_domain
 
     def load(self, ids, set_cursor=True, modified=False):
-        self.tree_states.clear()
-        self.tree_states_done.clear()
         self.group.load(ids, modified=modified)
         self.current_view.reset()
         if ids and self.current_view.view_type != 'calendar':
@@ -934,14 +934,16 @@ class Screen(SignalEvent):
         view.set_value()
         self.set_cursor(reset_view=False)
         if view.view_type == 'tree' and len(self.group):
-            start, end = view.treeview.get_visible_range()
-            vadjustment = view.treeview.get_vadjustment()
-            vadjustment.props.value = min(
-                vadjustment.props.value + vadjustment.props.page_increment,
-                vadjustment.props.upper)
-            model = view.treeview.get_model()
-            iter_ = model.get_iter(end)
-            self.current_record = model.get_value(iter_, 0)
+            range_ = view.treeview.get_visible_range()
+            if range_:
+                start, end = range_
+                vadjustment = view.treeview.get_vadjustment()
+                vadjustment.props.value = min(
+                    vadjustment.props.value + vadjustment.props.page_increment,
+                    vadjustment.props.upper)
+                model = view.treeview.get_model()
+                iter_ = model.get_iter(end)
+                self.current_record = model.get_value(iter_, 0)
         elif (view.view_type == 'form'
                 and self.current_record
                 and self.current_record.group):
@@ -1005,14 +1007,16 @@ class Screen(SignalEvent):
         view.set_value()
         self.set_cursor(reset_view=False)
         if view.view_type == 'tree' and len(self.group):
-            start, end = view.treeview.get_visible_range()
-            vadjustment = view.treeview.get_vadjustment()
-            vadjustment.props.value = min(
-                vadjustment.props.value - vadjustment.props.page_increment,
-                vadjustment.props.lower)
-            model = view.treeview.get_model()
-            iter_ = model.get_iter(start)
-            self.current_record = model.get_value(iter_, 0)
+            range_ = view.treeview.get_visible_range()
+            if range_:
+                start, end = range_
+                vadjustment = view.treeview.get_vadjustment()
+                vadjustment.props.value = min(
+                    vadjustment.props.value - vadjustment.props.page_increment,
+                    vadjustment.props.lower)
+                model = view.treeview.get_model()
+                iter_ = model.get_iter(start)
+                self.current_record = model.get_value(iter_, 0)
         elif (view.view_type == 'form'
                 and self.current_record
                 and self.current_record.group):
@@ -1092,6 +1096,9 @@ class Screen(SignalEvent):
     def clear(self):
         self.current_record = None
         self.group.clear()
+        self.tree_states_done.clear()
+        for view in self.views:
+            view.reset()
 
     def on_change(self, fieldname, attr):
         self.current_record.on_change(fieldname, attr)
