@@ -1,14 +1,13 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
-import tryton.rpc as rpc
-from tryton.common import message, selection, file_open, mailto
-from tryton.gui.window import Window
-from tryton.pyson import PYSONDecoder
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import gettext
-import tempfile
-import os
 import webbrowser
-from tryton.common import RPCProgress, RPCExecute, RPCException, slugify
+
+import tryton.rpc as rpc
+from tryton.common import RPCProgress, RPCExecute, RPCException
+from tryton.common import message, selection, file_write, file_open, mailto
+from tryton.config import CONFIG
+from tryton.pyson import PYSONDecoder
 
 _ = gettext.gettext
 
@@ -38,12 +37,8 @@ class Action(object):
         (type, data, print_p, name) = res
         if not print_p and direct_print:
             print_p = True
-        dtemp = tempfile.mkdtemp(prefix='tryton_')
 
-        fp_name = os.path.join(dtemp,
-            slugify(name) + os.extsep + slugify(type))
-        with open(fp_name, 'wb') as file_d:
-            file_d.write(data)
+        fp_name = file_write((name, type), data)
         if email_print:
             mailto(to=email.get('to'), cc=email.get('cc'),
                 subject=email.get('subject'), body=email.get('body'),
@@ -53,37 +48,32 @@ class Action(object):
         return True
 
     @staticmethod
-    def execute(act_id, data, action_type=None, context=None):
-        def get_action_type(actions):
-            try:
-                action, = actions()
-            except RPCException:
-                return
-            action_type = action['type']
-            exec_action(action_type)
-
-        def exec_action(action_type):
-            def callback(actions):
-                try:
-                    action, = actions()
-                except RPCException:
-                    return
-                Action._exec_action(action, data)
-
-            RPCExecute('model', action_type, 'search_read',
-                [('action', '=', act_id)], 0, 1, None, None,
-                context=context, callback=callback)
-
+    def execute(act_id, data, action_type=None, context=None, keyword=False):
+        # Must be executed synchronously to avoid double execution
+        # on double click.
         if not action_type:
-            RPCExecute('model', 'ir.action', 'read', [act_id],
-                ['type'], context=context, callback=get_action_type)
-        else:
-            exec_action(action_type)
+            action, = RPCExecute('model', 'ir.action', 'read', [act_id],
+                ['type'], context=context)
+            action_type = action['type']
+        action, = RPCExecute('model', action_type, 'search_read',
+            [('action', '=', act_id)], 0, 1, None, None,
+            context=context)
+        if keyword:
+            keywords = {
+                'ir.action.report': 'form_report',
+                'ir.action.wizard': 'form_action',
+                'ir.action.act_window': 'form_relate',
+                }
+            action.setdefault('keyword', keywords.get(action_type, ''))
+        Action._exec_action(action, data, context=context)
 
     @staticmethod
     def _exec_action(action, data=None, context=None):
+        from tryton.gui.window import Window
         if context is None:
             context = {}
+        else:
+            context = context.copy()
         if data is None:
             data = {}
         else:
@@ -91,9 +81,25 @@ class Action(object):
         if 'type' not in (action or {}):
             return
 
+        context.pop('active_id', None)
+        context.pop('active_ids', None)
+        context.pop('active_model', None)
+
+        def add_name_suffix(name, context=None):
+            if not data.get('ids') or not data.get('model'):
+                return name
+            max_records = 5
+            rec_names = RPCExecute('model', data['model'],
+                'read', data['ids'][:max_records], ['rec_name'],
+                context=context)
+            name_suffix = _(', ').join([x['rec_name'] for x in rec_names])
+            if len(data['ids']) > max_records:
+                name_suffix += _(',\u2026')
+            return _('%s (%s)') % (name, name_suffix)
+
         data['action_id'] = action['id']
         if action['type'] == 'ir.action.act_window':
-            view_ids = False
+            view_ids = []
             view_mode = None
             if action.get('views', []):
                 view_ids = [x[0] for x in action['views']]
@@ -105,41 +111,57 @@ class Action(object):
             ctx = {
                 'active_model': data.get('model'),
                 'active_id': data.get('id'),
-                'active_ids': data.get('ids', []),
+                'active_ids': data.get('ids') or [],
             }
             ctx.update(rpc.CONTEXT)
             ctx['_user'] = rpc._USER
             decoder = PYSONDecoder(ctx)
-            action_ctx = decoder.decode(action.get('pyson_context') or '{}')
+            action_ctx = context.copy()
+            action_ctx.update(
+                decoder.decode(action.get('pyson_context') or '{}'))
             ctx.update(action_ctx)
             ctx.update(context)
-            action_ctx.update(context)
 
             ctx['context'] = ctx
             decoder = PYSONDecoder(ctx)
             domain = decoder.decode(action['pyson_domain'])
             order = decoder.decode(action['pyson_order'])
             search_value = decoder.decode(action['pyson_search_value'] or '[]')
-            tab_domain = [(n, decoder.decode(d)) for n, d in action['domains']]
+            tab_domain = [(n, decoder.decode(d), c)
+                for n, d, c in action['domains']]
 
-            name = False
-            if action.get('window_name', True):
-                name = action.get('name', False)
+            name = action.get('name', '')
+            if action.get('keyword', ''):
+                name = add_name_suffix(name, action_ctx)
 
             res_model = action.get('res_model', data.get('res_model'))
             res_id = action.get('res_id', data.get('res_id'))
+            limit = action.get('limit')
+            if limit is None:
+                limit = CONFIG['client.limit']
 
-            Window.create(view_ids, res_model, res_id, domain,
-                    action_ctx, order, view_mode, name=name,
-                    limit=action.get('limit'),
-                    search_value=search_value,
-                    icon=(action.get('icon.rec_name') or ''),
-                    tab_domain=tab_domain)
+            Window.create(res_model,
+                view_ids=view_ids,
+                res_id=res_id,
+                domain=domain,
+                context=action_ctx,
+                order=order,
+                mode=view_mode,
+                name=name,
+                limit=limit,
+                search_value=search_value,
+                icon=(action.get('icon.rec_name') or ''),
+                tab_domain=tab_domain,
+                context_model=action['context_model'],
+                context_domain=action['context_domain'])
         elif action['type'] == 'ir.action.wizard':
+            name = action.get('name', '')
+            if action.get('keyword', 'form_action') == 'form_action':
+                name = add_name_suffix(name, context)
             Window.create_wizard(action['wiz_name'], data,
                 direct_print=action.get('direct_print', False),
                 email_print=action.get('email_print', False),
-                email=action.get('email'), name=action.get('name', False),
+                email=action.get('email'), name=name,
                 context=context, icon=(action.get('icon.rec_name') or ''),
                 window=action.get('window', False))
 
@@ -174,7 +196,7 @@ class Action(object):
             Action._exec_action(action, data, context=context)
             return (name, action)
         elif not len(keyact) and warning:
-            message(_('No action defined!'))
+            message(_('No action defined.'))
         return False
 
     @staticmethod
